@@ -500,12 +500,15 @@ nfc_initiator_init_secure_element(nfc_device *pnd)
  *
  * @param pnd \a nfc_device struct pointer that represent currently used device
  * @param nm desired modulation
- * @param pbtInitData optional initiator data used for Felica, ISO14443B, Topaz polling or to select a specific UID in ISO14443A.
+ * @param pbtInitData optional initiator data, NULL for using the default values.
  * @param szInitData length of initiator data \a pbtInitData.
  * @note pbtInitData is used with different kind of data depending on modulation type:
  * - for an ISO/IEC 14443 type A modulation, pbbInitData contains the UID you want to select;
- * - for an ISO/IEC 14443 type B modulation, pbbInitData contains Application Family Identifier (AFI) (see ISO/IEC 14443-3);
- * - for a FeliCa modulation, pbbInitData contains polling payload (see ISO/IEC 18092 11.2.2.5).
+ * - for an ISO/IEC 14443 type B modulation, pbbInitData contains Application Family Identifier (AFI) (see ISO/IEC 14443-3)
+        and optionally a second byte = 0x01 if you want to use probabilistic approach instead of timeslot approach;
+ * - for a FeliCa modulation, pbbInitData contains a 5-byte polling payload (see ISO/IEC 18092 11.2.2.5).
+ * - for ISO14443B', ASK CTx and ST SRx, see corresponding standards
+ * - if NULL, default values adequate for the chosen modulation will be used.
  *
  * @param[out] pnt \a nfc_target struct pointer which will filled if available
  *
@@ -520,24 +523,19 @@ nfc_initiator_select_passive_target(nfc_device *pnd,
                                     const uint8_t *pbtInitData, const size_t szInitData,
                                     nfc_target *pnt)
 {
-  uint8_t  abtInit[MAX(12, szInitData)];
-  size_t  szInit;
-
-  switch (nm.nmt) {
-    case NMT_ISO14443A:
-      iso14443_cascade_uid(pbtInitData, szInitData, abtInit, &szInit);
-      break;
-
-    case NMT_JEWEL:
-    case NMT_ISO14443B:
-    case NMT_ISO14443BI:
-    case NMT_ISO14443B2SR:
-    case NMT_ISO14443B2CT:
-    case NMT_FELICA:
-    case NMT_DEP:
-      memcpy(abtInit, pbtInitData, szInitData);
-      szInit = szInitData;
-      break;
+  uint8_t *abtInit = NULL;
+  uint8_t abtTmpInit[MAX(12, szInitData)];
+  size_t  szInit = 0;
+  if (szInitData == 0) {
+    // Provide default values, if any
+    prepare_initiator_data(nm, &abtInit, &szInit);
+  } else if (nm.nmt == NMT_ISO14443A) {
+    abtInit = abtTmpInit;
+    iso14443_cascade_uid(pbtInitData, szInitData, abtInit, &szInit);
+  } else {
+    abtInit = abtTmpInit;
+    memcpy(abtInit, pbtInitData, szInitData);
+    szInit = szInitData;
   }
 
   HAL(initiator_select_passive_target, pnd, nm, abtInit, szInit, pnt);
@@ -573,6 +571,7 @@ nfc_initiator_list_passive_targets(nfc_device *pnd,
   pnd->last_error = 0;
 
   // Let the reader only try once to find a tag
+  bool bInfiniteSelect = pnd->bInfiniteSelect;
   if ((res = nfc_device_set_property_bool(pnd, NP_INFINITE_SELECT, false)) < 0) {
     return res;
   }
@@ -603,6 +602,11 @@ nfc_initiator_list_passive_targets(nfc_device *pnd,
       break;
     }
   }
+  if (bInfiniteSelect) {
+    if ((res = nfc_device_set_property_bool(pnd, NP_INFINITE_SELECT, true)) < 0) {
+      return res;
+    }
+  }
   return szTargetFound;
 }
 
@@ -684,18 +688,30 @@ nfc_initiator_poll_dep_target(struct nfc_device *pnd,
   const int period = 300;
   int remaining_time = timeout;
   int res;
+  int result = 0;
+  bool bInfiniteSelect = pnd->bInfiniteSelect;
   if ((res = nfc_device_set_property_bool(pnd, NP_INFINITE_SELECT, true)) < 0)
     return res;
   while (remaining_time > 0) {
     if ((res = nfc_initiator_select_dep_target(pnd, ndm, nbr, pndiInitiator, pnt, period)) < 0) {
-      if (res != NFC_ETIMEOUT)
-        return res;
+      if (res != NFC_ETIMEOUT) {
+        result = res;
+        goto end;
+      }
+    }
+    if (res == 1) {
+      result = res;
+      goto end;
     }
-    if (res == 1)
-      return res;
     remaining_time -= period;
   }
-  return 0;
+end:
+  if (! bInfiniteSelect) {
+    if ((res = nfc_device_set_property_bool(pnd, NP_INFINITE_SELECT, false)) < 0) {
+      return res;
+    }
+  }
+  return result;
 }
 
 /** @ingroup initiator
@@ -836,7 +852,9 @@ nfc_initiator_transceive_bytes_timed(nfc_device *pnd,
  * @brief Check target presence
  * @return Returns 0 on success, otherwise returns libnfc's error code.
  *
- * This function tests if \a nfc_target is currently present on NFC device.
+ * @param pnd \a nfc_device struct pointer that represent currently used device
+ * @param pnt a \a nfc_target struct pointer where desired target information was stored (optionnal, can be \e NULL).
+ * This function tests if \a nfc_target (or last selected tag if \e NULL) is currently present on NFC device.
  * @warning The target have to be selected before check its presence
  * @warning To run the test, one or more commands will be sent to target
 */
